from datetime import datetime
from typing import List, Optional, Union, overload

from ....swagger.models import (
    ModelValidationMethod,
    NlpLanguage,
    ProblemType,
    RegressionScorers,
)
from ....utils.typing import UNSET, PathLike, Unset
from ...connection import ConnectionLike, from_connectionlike
from ...data_set import DataLike
from ..serialization import parse_model_from_path
from ..shared import AnalyzeModel, BaseModel, RemoteModel
from .local import InitialConfig, LocalRegressor


def _default_model_name() -> str:
    return f"Regression Model from Python Client {datetime.utcnow().strftime('%Y/%m/%d %H:%M:%S')}"


class AnalyzeRegressor(AnalyzeModel):
    @overload
    def __init__(
        self,
        scorer: RegressionScorers = RegressionScorers.RMSE,
        model_validation_method: ModelValidationMethod = ModelValidationMethod.FIVE_FOLD_CROSS,
        name: Optional[str] = None,
        description: Optional[str] = None,
        number_of_evaluations: int = 300,
        nlp_language: NlpLanguage = NlpLanguage.AUTO_DETECT,
        draft_mode: bool = False,
        *,
        connection: ConnectionLike,
    ):
        """
        Create a new Analyze regression model
        :param scorer: The scorer to use to determine the optimal model
        :param model_validation_method: The method to use to validate the model
        :param name: A name for the model
        :param description: A description for the model
        :param number_of_evaluations: The number of models to evaluate during optimization
        :param nlp_language: The language used when performing NLP on text columns
        :param draft_mode: Create a draft model rather than a fully optimized model
        :param connection: The connection to the Analyze project in which the model will be created
        """
        ...

    @overload
    def __init__(self, *, _internal_model: BaseModel):
        """Internal constructor - DO NOT USE"""
        ...

    def __init__(
        self,
        scorer: RegressionScorers = RegressionScorers.RMSE,
        model_validation_method: ModelValidationMethod = ModelValidationMethod.FIVE_FOLD_CROSS,
        name: Optional[str] = None,
        description: Optional[str] = None,
        number_of_evaluations: int = 300,
        nlp_language: NlpLanguage = NlpLanguage.AUTO_DETECT,
        draft_mode: bool = False,
        *,
        connection: Union[ConnectionLike, Unset] = UNSET,
        _internal_model: Union[BaseModel, Unset] = UNSET,
    ):
        """Internal constructor - DO NOT USE"""
        if not isinstance(_internal_model, Unset):
            # Default to using the regressor if it was specified
            super().__init__(_internal_model=_internal_model)
        else:
            # Use the user parameters
            if isinstance(connection, Unset):
                raise Exception(
                    "You must specify either the connection or the _regressor parameter."
                )
            initial_config = InitialConfig(
                scorer=scorer,
                model_validation_method=model_validation_method,
                name=name if name is not None else _default_model_name(),
                description=description,
                number_of_evaluations=number_of_evaluations,
                nlp_language=nlp_language,
                draft_mode=draft_mode,
            )
            super().__init__(_internal_model=LocalRegressor(connection, initial_config))

    @classmethod
    def load_model(
        cls, path: PathLike, *, connection: ConnectionLike
    ) -> "AnalyzeRegressor":
        """
        Load a regression model from a saved file
        :param path: The path of the file to load
        :param connection: A connection to the project that contains the model
        :return:
        """
        connection = from_connectionlike(connection)
        serialized_model = parse_model_from_path(path)
        remote_model = RemoteModel.from_serialized_model(connection, serialized_model)

        if remote_model.model_info().problem_type != ProblemType.REGRESSION:
            raise Exception("The model is not an Analyze regression model.")

        return AnalyzeRegressor(_internal_model=remote_model)

    @classmethod
    def load_model_from_server(
        cls, model_id: int, *, connection: ConnectionLike
    ) -> "AnalyzeRegressor":
        """
        Load a regression model from the Analyze server
        :param model_id: The model ID
        :param connection: A connection to the project that contains the model
        :return: A fitted regressor that uses the server model to perform predictions
        """
        return AnalyzeRegressor(_internal_model=RemoteModel(connection, model_id))

    @overload
    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        wait_until_complete: bool = True,
    ) -> "AnalyzeRegressor":
        """Fit the regression model using random data partitioning"""
        ...

    @overload
    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        wait_until_complete: bool = True,
        # Regression specific fit parameters
        order_by: str,
    ) -> "AnalyzeRegressor":
        """Fit the regression model using ordered data partitioning"""
        ...

    @overload
    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        wait_until_complete: bool = True,
        # Regression specific fit parameters
        partition_by: str,
    ) -> "AnalyzeRegressor":
        """Fit the regression model using manual data partitioning"""
        ...

    @overload
    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        wait_until_complete: bool = True,
        # Regression specific fit parameters
        no_mixing: List[str],
    ) -> "AnalyzeRegressor":
        """Fit the regression model using a no-mixing column data partitioning"""
        ...

    def fit(
        self,
        data: DataLike,
        target: str,
        excluded_columns: Optional[List[str]] = None,
        *,
        wait_until_complete: bool = True,
        # Regression specific fit parameters
        order_by: Optional[str] = None,
        partition_by: Optional[str] = None,
        no_mixing: Optional[List[str]] = None,
    ) -> "AnalyzeRegressor":
        """Internal method - DO NOT USE"""
        # Fit must explicitly override the model with the new fitted model
        super()._fit(
            data,
            target,
            excluded_columns,
            wait_until_complete=wait_until_complete,
            order_by=order_by,
            partition_by=partition_by,
            no_mixing=no_mixing,
        )
        return self



import pandas as pd
import requests
from httpx import BasicAuth

from mindfoundry.client.analyze.project_api_swagger_client import (
    AnalyzeSwaggerClientWrapper,
)
from mindfoundry.client.analyze.swagger import Client
from mindfoundry.client.analyze.utils import (
    RefreshableAccessToken,
    TokenAuthenticatedClient,
)


def create_project_api_swagger_client(
    base_url: str, client_id: str, client_secret: str
) -> Client:
    """
    Create a client for calling the Analyze Project API

    e.g.
    >>> base_url = "https://example.app.mindfoundry.ai/"
    >>> analyze_client = create_project_api_client(base_url, "<client_id>", "<client_secret>")
    >>> data = pd.DataFrame({"foo": [1, 2], "bar": [3, 4]})
    >>> model_api = analyze_client.create_file_data_set(data, "Data Set Name")

    :param base_url: the base url for Analyze
    :param client_id: the client id for the Project API
    :param client_secret: the client secret for the Project API
    """
    auth_token_url = base_url.strip("/") + "/api/auth/token"
    api_url = base_url.strip("/") + "/api"
    client = TokenAuthenticatedClient(
        base_url=api_url,
        token_generator=RefreshableAccessToken(
            auth_token_url=auth_token_url,
            credentials=BasicAuth(client_id, client_secret),
        ),
        timeout=60,
    )
    return client


def create_project_api_client(
    base_url: str, client_id: str, client_secret: str
) -> AnalyzeSwaggerClientWrapper:
    """
    Create a client for calling the Analyze Project API

    e.g.
    >>> base_url = "https://example.app.mindfoundry.ai/"
    >>> analyze_client = create_project_api_client(base_url, "<client_id>", "<client_secret>")
    >>> data = pd.DataFrame({"foo": [1, 2], "bar": [3, 4]})
    >>> model_api = analyze_client.create_file_data_set(data, "Data Set Name")

    :param base_url: the base url for Analyze
    :param client_id: the client id for the Project API
    :param client_secret: the client secret for the Project API
    """
    client = create_project_api_swagger_client(base_url, client_id, client_secret)
    return AnalyzeSwaggerClientWrapper(client)


class PublishedApi:
    def __init__(
        self, base_url: str, api_endpoint: str, client_id: str, client_secret: str
    ):
        """
        Create a client for calling a published model api.

        e.g.
        >>> base_url = "https://example.app.mindfoundry.ai"
        >>> analyze_client = create_project_api_client(base_url, "client_id", "client_secret")
        >>> model_api_id = analyze_client.create_model_api(123, "new api")
        >>> model_api = analyze_client.get_model_api(model_api_id)
        >>> client_credentials = analyze_client.create_model_api_client(model_api.id, "credentials for new api")
        >>>
        >>> model_api_client = PublishedApi(base_url, model_api.api_endpoint, client_credentials.id, client_credentials.secret))  # pylint: disable=line-too-long
        >>> features = pd.DataFrame(...)
        >>> predictions = model_api_client.transform(features)

        :param base_url: the same base url used for the analyze client
        :param api_endpoint: the 'apiEndpoint' value from AnalyzeClient.create_model_api response
        :param client_id: the 'id' value from AnalyzeClient.create_api_client response
        :param client_secret: the 'secret' value from AnalyzeClient.create_api_client response
        """
        self._base_url = base_url.strip("/")
        self._token_generator = RefreshableAccessToken(
            auth_token_url=self._base_url + "/api/auth/token",
            credentials=BasicAuth(client_id, client_secret),
        )
        self._api_endpoint = api_endpoint

    def transform(self, features: pd.DataFrame):
        payload = {
            "records": features.to_dict("records"),
        }

        response = self._request("POST", self._api_endpoint, json=payload)

        return response.json()

    def _request(self, *args, **kwargs):
        response = requests.request(
            *args,
            headers={"Authorization": "Bearer " + self._token_generator.get_token()},
            **kwargs
        )
        if response.ok:
            return response
        else:
            raise Exception(response.text)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.set_column_type_step_step_type import SetColumnTypeStepStepType
from ..types import UNSET, Unset

T = TypeVar("T", bound="SetColumnTypeStep")

@attr.s(auto_attribs=True)
class SetColumnTypeStep:
    """  """
    columns: List[str]
    new_type: str
    step_type: SetColumnTypeStepStepType
    epoch: Union[Unset, str] = UNSET
    invalid: Union[Unset, str] = UNSET
    not_int: Union[Unset, str] = UNSET
    default_not_int: Union[Unset, str] = UNSET
    datetime_format: Union[Unset, str] = UNSET
    default_value: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        columns = self.columns




        new_type =  self.new_type
        step_type = self.step_type.value

        epoch =  self.epoch
        invalid =  self.invalid
        not_int =  self.not_int
        default_not_int =  self.default_not_int
        datetime_format =  self.datetime_format
        default_value =  self.default_value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "columns": columns,
            "newType": new_type,
            "stepType": step_type,
        })
        if epoch is not UNSET:
            field_dict["epoch"] = epoch
        if invalid is not UNSET:
            field_dict["invalid"] = invalid
        if not_int is not UNSET:
            field_dict["notInt"] = not_int
        if default_not_int is not UNSET:
            field_dict["defaultNotInt"] = default_not_int
        if datetime_format is not UNSET:
            field_dict["datetimeFormat"] = datetime_format
        if default_value is not UNSET:
            field_dict["defaultValue"] = default_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        columns = cast(List[str], d.pop("columns"))


        new_type = d.pop("newType")

        step_type = SetColumnTypeStepStepType(d.pop("stepType"))




        epoch = d.pop("epoch", UNSET)

        invalid = d.pop("invalid", UNSET)

        not_int = d.pop("notInt", UNSET)

        default_not_int = d.pop("defaultNotInt", UNSET)

        datetime_format = d.pop("datetimeFormat", UNSET)

        default_value = d.pop("defaultValue", UNSET)

        set_column_type_step = cls(
            columns=columns,
            new_type=new_type,
            step_type=step_type,
            epoch=epoch,
            invalid=invalid,
            not_int=not_int,
            default_not_int=default_not_int,
            datetime_format=datetime_format,
            default_value=default_value,
        )

        set_column_type_step.additional_properties = d
        return set_column_type_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict

from attr import attrs

from mindfoundry.client.analyze.swagger import Client
from mindfoundry.client.analyze.utils._refreshable_access_token import (
    RefreshableAccessToken,
)


@attrs(auto_attribs=True)
class TokenAuthenticatedClient(Client):
    """A client for the swagger API that adds an authentication token to the request using the provided token generator"""

    token_generator: RefreshableAccessToken

    def get_headers(self) -> Dict[str, str]:
        """ Get headers to be used in authenticated endpoints """
        return {
            "Authorization": f"Bearer {self.token_generator.get_token()}",
            **self.headers,
        }



from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateTestRequest")

@attr.s(auto_attribs=True)
class CreateTestRequest:
    """  """
    model_id: int
    data_set_id: int
    name: Union[Unset, str] = UNSET
    description: Union[Unset, Optional[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        model_id =  self.model_id
        data_set_id =  self.data_set_id
        name =  self.name
        description =  self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "modelId": model_id,
            "dataSetId": data_set_id,
        })
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        model_id = d.pop("modelId")

        data_set_id = d.pop("dataSetId")

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        create_test_request = cls(
            model_id=model_id,
            data_set_id=data_set_id,
            name=name,
            description=description,
        )

        create_test_request.additional_properties = d
        return create_test_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.discriminators import Discriminators
from ..models.simple_forecast import SimpleForecast

T = TypeVar("T", bound="PredictionForecastResponseModelForecastsItem")

@attr.s(auto_attribs=True)
class PredictionForecastResponseModelForecastsItem:
    """  """
    discriminators: Discriminators
    simple_forecasts: List[SimpleForecast]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        discriminators = self.discriminators.to_dict()

        simple_forecasts = []
        for simple_forecasts_item_data in self.simple_forecasts:
            simple_forecasts_item = simple_forecasts_item_data.to_dict()

            simple_forecasts.append(simple_forecasts_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "discriminators": discriminators,
            "simpleForecasts": simple_forecasts,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        discriminators = Discriminators.from_dict(d.pop("discriminators"))




        simple_forecasts = []
        _simple_forecasts = d.pop("simpleForecasts")
        for simple_forecasts_item_data in (_simple_forecasts):
            simple_forecasts_item = SimpleForecast.from_dict(simple_forecasts_item_data)



            simple_forecasts.append(simple_forecasts_item)


        prediction_forecast_response_model_forecasts_item = cls(
            discriminators=discriminators,
            simple_forecasts=simple_forecasts,
        )

        prediction_forecast_response_model_forecasts_item.additional_properties = d
        return prediction_forecast_response_model_forecasts_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ClassificationScorers(str, Enum):
    F1_WEIGHTED_AVG = "F1_WEIGHTED_AVG"
    ACCURACY = "ACCURACY"
    ROC_AUC = "ROC_AUC"
    F1_SINGLE_CLASS = "F1_SINGLE_CLASS"
    RECALL_SINGLE_CLASS = "RECALL_SINGLE_CLASS"
    PRECISION_SINGLE_CLASS = "PRECISION_SINGLE_CLASS"

    def __str__(self) -> str:
        return str(self.value)


from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.cadence import Cadence
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateForecastingRequest")

@attr.s(auto_attribs=True)
class CreateForecastingRequest:
    """  """
    name: str
    data_set_id: int
    target_column: str
    time_index_column: str
    cadence: Cadence
    horizons: List[int]
    description: Union[Unset, str] = UNSET
    excluded_columns: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        name =  self.name
        data_set_id =  self.data_set_id
        target_column =  self.target_column
        time_index_column =  self.time_index_column
        cadence = self.cadence.value

        horizons = self.horizons




        description =  self.description
        excluded_columns: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.excluded_columns, Unset):
            excluded_columns = self.excluded_columns





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "dataSetId": data_set_id,
            "targetColumn": target_column,
            "timeIndexColumn": time_index_column,
            "cadence": cadence,
            "horizons": horizons,
        })
        if description is not UNSET:
            field_dict["description"] = description
        if excluded_columns is not UNSET:
            field_dict["excludedColumns"] = excluded_columns

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        data_set_id = d.pop("dataSetId")

        target_column = d.pop("targetColumn")

        time_index_column = d.pop("timeIndexColumn")

        cadence = Cadence(d.pop("cadence"))




        horizons = cast(List[int], d.pop("horizons"))


        description = d.pop("description", UNSET)

        excluded_columns = cast(List[str], d.pop("excludedColumns", UNSET))


        create_forecasting_request = cls(
            name=name,
            data_set_id=data_set_id,
            target_column=target_column,
            time_index_column=time_index_column,
            cadence=cadence,
            horizons=horizons,
            description=description,
            excluded_columns=excluded_columns,
        )

        create_forecasting_request.additional_properties = d
        return create_forecasting_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from time import sleep
from typing import Optional

from ..swagger import Client
from ..swagger import models as dto
from ..swagger.api.models import get_model
from ..swagger.api.other import get_future
from ..swagger.api.predictions import get_prediction
from ..swagger.api.tests import get_test
from ..swagger.types import Response
from ..utils._swagger_response_utils import (
    check_exists,
    check_not_unset,
    check_response,
)


def wait_for_future_to_succeed(
    client: Client, response: Response[Optional[dto.Future]]
) -> dto.FutureOutputofyourrequest:
    status = check_response(response).status
    future_id = check_not_unset(check_response(response).future_id)

    while status == dto.Status.IN_PROGRESS:
        sleep(5)
        response = get_future.sync_detailed(client=client, future_id=future_id)
        status = check_response(response).status

    if status == dto.Status.SUCCEEDED:
        return check_exists(check_not_unset(check_response(response).response))
    else:
        raise Exception(response.content)


def wait_for_model_to_complete(client: Client, model_id: int) -> int:
    while True:
        model_response = check_exists(get_model.sync(client=client, id=model_id))
        status = model_response.status
        if status == dto.ModelStatus.COMPLETE:
            return model_response.id
        if status == dto.ModelStatus.FAILED:
            raise Exception(model_response.failure_details)
        sleep(5)


def wait_for_prediction_to_complete(client: Client, prediction_id: int) -> int:
    while True:
        prediction_response = check_response(
            get_prediction.sync_detailed(client=client, id=prediction_id)
        )
        status = prediction_response.status
        if status != dto.Status.IN_PROGRESS:
            break
        sleep(10)

    if status == dto.Status.SUCCEEDED:
        return prediction_response.id
    else:
        raise Exception(prediction_response.failure_details)


def wait_for_test_to_complete(client: Client, test_id: int) -> int:
    while True:
        test_response = check_response(
            get_test.sync_detailed(client=client, id=test_id)
        )
        status = test_response.status
        if status != dto.Status.IN_PROGRESS:
            break
        sleep(10)

    if status == dto.Status.SUCCEEDED:
        return test_response.id
    else:
        raise Exception(test_response.failure_details)



from enum import Enum


class Status(str, Enum):
    IN_PROGRESS = "IN_PROGRESS"
    SUCCEEDED = "SUCCEEDED"
    FAILED = "FAILED"
    MISSING = "MISSING"

    def __str__(self) -> str:
        return str(self.value)


from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.deployed_model_api_response import DeployedModelApiResponse
from ..types import UNSET, Unset

T = TypeVar("T", bound="DeployedModelApiListResponse")

@attr.s(auto_attribs=True)
class DeployedModelApiListResponse:
    """  """
    published_models: Union[Unset, List[DeployedModelApiResponse]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        published_models: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.published_models, Unset):
            published_models = []
            for published_models_item_data in self.published_models:
                published_models_item = published_models_item_data.to_dict()

                published_models.append(published_models_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if published_models is not UNSET:
            field_dict["publishedModels"] = published_models

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        published_models = []
        _published_models = d.pop("publishedModels", UNSET)
        for published_models_item_data in (_published_models or []):
            published_models_item = DeployedModelApiResponse.from_dict(published_models_item_data)



            published_models.append(published_models_item)


        deployed_model_api_list_response = cls(
            published_models=published_models,
        )

        deployed_model_api_list_response.additional_properties = d
        return deployed_model_api_list_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.filter_rows_rule import FilterRowsRule
from ..models.filter_rows_step_step_type import FilterRowsStepStepType

T = TypeVar("T", bound="FilterRowsStep")

@attr.s(auto_attribs=True)
class FilterRowsStep:
    """  """
    rules: List[FilterRowsRule]
    step_type: FilterRowsStepStepType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        rules = []
        for rules_item_data in self.rules:
            rules_item = rules_item_data.to_dict()

            rules.append(rules_item)




        step_type = self.step_type.value


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "rules": rules,
            "stepType": step_type,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        rules = []
        _rules = d.pop("rules")
        for rules_item_data in (_rules):
            rules_item = FilterRowsRule.from_dict(rules_item_data)



            rules.append(rules_item)


        step_type = FilterRowsStepStepType(d.pop("stepType"))




        filter_rows_step = cls(
            rules=rules,
            step_type=step_type,
        )

        filter_rows_step.additional_properties = d
        return filter_rows_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
isort:skip_file
"""
from ._version import get_versions

# The sklearn-style client
from . import sklearn_client

# The swagger API
from .project_api_swagger_client import AnalyzeSwaggerClientWrapper
from . import swagger
from .client import create_project_api_client

# The published model API
from .client import PublishedApi


# pylint: disable-all
_versions_dict = get_versions()
__version__ = _versions_dict["version"]
__gitsha__ = _versions_dict["full-revisionid"]
del get_versions, _versions_dict

from ._version import get_versions

__version__ = get_versions()["version"]
del get_versions



import abc
from typing import Generic, TypeVar, Union

from ..swagger import Client
from ..swagger.api.models import get_model
from ..swagger.api.predictions import get_prediction
from ..swagger.api.tests import get_test
from ..swagger.models import (
    ModelResponse,
    ModelStatus,
    PredictionResponse,
    Status,
    TestResponse,
)
from ._swagger_response_utils import check_exists
from .typing import UNSET, Unset

T = TypeVar("T")


class CachedResponse(Generic[T], abc.ABC):
    def __init__(self):
        self._cached_info: Union[Unset, T] = UNSET

    def get_response(self) -> T:
        self._cached_info = self._sync_info(self._cached_info)
        if isinstance(self._cached_info, Unset):
            raise Exception("Could not obtain information from remote server")
        return self._cached_info

    @abc.abstractmethod
    def _sync_info(self, current: Union[Unset, T]) -> Union[Unset, T]:
        """Get the latest version of the information"""


class CachedModelResponse(CachedResponse[ModelResponse]):
    def __init__(self, client: Client, model_id: int):
        super().__init__()
        self._client = client
        self._model_id = model_id

    def _sync_info(
        self, current: Union[Unset, ModelResponse]
    ) -> Union[Unset, ModelResponse]:
        # Get the info *unless* the previous info was already complete or failed
        if not isinstance(current, Unset) and current.status in (
            ModelStatus.COMPLETE,
            ModelStatus.FAILED,
        ):
            return current
        return check_exists(get_model.sync(client=self._client, id=self._model_id))


class CachedPredictionResponse(CachedResponse[PredictionResponse]):
    def __init__(self, client: Client, prediction_id: int):
        super().__init__()
        self._client = client
        self._prediction_id = prediction_id

    def _sync_info(
        self, current: Union[Unset, PredictionResponse]
    ) -> Union[Unset, PredictionResponse]:
        # Get the info *unless* the previous info was already complete or failed
        if not isinstance(current, Unset) and current.status in (
            Status.SUCCEEDED,
            Status.MISSING,
            Status.FAILED,
        ):
            return current
        return check_exists(
            get_prediction.sync(client=self._client, id=self._prediction_id)
        )


class CachedTestResponse(CachedResponse[TestResponse]):
    def __init__(self, client: Client, test_id: int):
        super().__init__()
        self._client = client
        self._test_id = test_id

    def _sync_info(
        self, current: Union[Unset, TestResponse]
    ) -> Union[Unset, TestResponse]:
        # Get the info *unless* the previous info was already complete or failed
        if not isinstance(current, Unset) and current.status in (
            Status.SUCCEEDED,
            Status.MISSING,
            Status.FAILED,
        ):
            return current
        return check_exists(get_test.sync(client=self._client, id=self._test_id))



from time import time

import httpx


class RefreshableAccessToken:
    def __init__(self, auth_token_url: str, credentials: httpx.Auth):
        """
        Get an OAuth access token that is currently valid

        :param auth_token_url: The address of the authentication token endpoint
        :param credentials: The credentials to use to access the authentication token endpoint
        """
        self.auth_token_url = auth_token_url
        self.credentials = credentials
        self._token_expiry_time = 0
        self._access_token = "invalid"

    def get_token(self) -> str:
        """Get a token that is currently valid, refreshing the token if necessary"""

        if self._token_expiry_time <= (int(time()) + 10):
            self._get_new_access_token()

        return self._access_token

    def _get_new_access_token(self):
        """ Get a new token from the access token endpoint using the supplied credentials"""
        response = httpx.post(
            self.auth_token_url,
            auth=self.credentials,
            data={"grant_type": "client_credentials"},
        )

        if response.status_code == httpx.codes.CREATED:
            self._access_token = response.json()["access_token"]
            self._token_expiry_time = int(time()) + int(response.json()["expires_in"])
        else:
            raise Exception(response.text)



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.data_set_response import DataSetResponse

T = TypeVar("T", bound="DataSetsResponse")

@attr.s(auto_attribs=True)
class DataSetsResponse:
    """  """
    data_sets: List[DataSetResponse]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        data_sets = []
        for data_sets_item_data in self.data_sets:
            data_sets_item = data_sets_item_data.to_dict()

            data_sets.append(data_sets_item)





        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "dataSets": data_sets,
        })

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        data_sets = []
        _data_sets = d.pop("dataSets")
        for data_sets_item_data in (_data_sets):
            data_sets_item = DataSetResponse.from_dict(data_sets_item_data)



            data_sets.append(data_sets_item)


        data_sets_response = cls(
            data_sets=data_sets,
        )

        data_sets_response.additional_properties = d
        return data_sets_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import os

from setuptools import find_namespace_packages, setup

import versioneer

install_requires = [
    "attrs",
    "httpx",
    "pandas",
    "pyarrow",
    "requests",
]  # etc

dev_extras = [
    "black",
    "check-manifest",
    "coverage",
    "freezegun",
    "isort",
    "mypy",
    "pre-commit",
    "pylint",
    "pytest",
    "pytest-freezegun",
    "pytest-httpx",
    "pylint-pytest",
    "requests_toolbelt",
    "tox",
]

extras_require = {"dev": dev_extras}


# Add a `pip install .[all]` target:
all_extras = set()
for extras_list in extras_require.values():
    all_extras.update(extras_list)
extras_require["all"] = list(all_extras)

version = versioneer.get_version()

project_repo_dir = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the README file
with open(os.path.join(project_repo_dir, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="mindfoundry.client.analyze",
    license="MIT",
    version=version,
    cmdclass=versioneer.get_cmdclass(),
    description="Mind Foundry Analyze Python Client",
    long_description=long_description,
    # The project"s main homepage.
    url="https://www.mindfoundry.ai/platform",
    # Author details
    author="Mind Foundry Ltd",
    author_email="support@mindfoundry.ai",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",

        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_namespace_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    package_data={},
    install_requires=install_requires,
    extras_require=extras_require,
    namespace_packages=["mindfoundry"],
)



import datetime
from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.data_prep_step_response import DataPrepStepResponse
from ..models.status import Status
from ..types import UNSET, Unset

T = TypeVar("T", bound="DataSetResponse")

@attr.s(auto_attribs=True)
class DataSetResponse:
    """  """
    id: int
    project_id: int
    name: str
    rows: int
    columns: int
    created_at: datetime.datetime
    created_by: str
    data_prep_steps: List[DataPrepStepResponse]
    status: Status
    description: Union[Unset, Optional[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        id =  self.id
        project_id =  self.project_id
        name =  self.name
        rows =  self.rows
        columns =  self.columns
        created_at = self.created_at.isoformat()

        created_by =  self.created_by
        data_prep_steps = []
        for data_prep_steps_item_data in self.data_prep_steps:
            data_prep_steps_item = data_prep_steps_item_data.to_dict()

            data_prep_steps.append(data_prep_steps_item)




        status = self.status.value

        description =  self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "projectId": project_id,
            "name": name,
            "rows": rows,
            "columns": columns,
            "createdAt": created_at,
            "createdBy": created_by,
            "dataPrepSteps": data_prep_steps,
            "status": status,
        })
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        project_id = d.pop("projectId")

        name = d.pop("name")

        rows = d.pop("rows")

        columns = d.pop("columns")

        created_at = isoparse(d.pop("createdAt"))




        created_by = d.pop("createdBy")

        data_prep_steps = []
        _data_prep_steps = d.pop("dataPrepSteps")
        for data_prep_steps_item_data in (_data_prep_steps):
            data_prep_steps_item = DataPrepStepResponse.from_dict(data_prep_steps_item_data)



            data_prep_steps.append(data_prep_steps_item)


        status = Status(d.pop("status"))




        description = d.pop("description", UNSET)

        data_set_response = cls(
            id=id,
            project_id=project_id,
            name=name,
            rows=rows,
            columns=columns,
            created_at=created_at,
            created_by=created_by,
            data_prep_steps=data_prep_steps,
            status=status,
            description=description,
        )

        data_set_response.additional_properties = d
        return data_set_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import abc
from typing import Optional

from ....swagger.models import ModelResponse
from ....utils.typing import PathLike
from ...data_set import DataLike
from ...prediction import Prediction
from ...test import Test
from .base import BaseModel


class LocalModel(BaseModel, abc.ABC):
    """
    A model that has not yet been saved to Analyze (and thus hasn't been fitted)
    """

    def is_fitting(self) -> bool:
        return False

    def is_fitted(self) -> bool:
        return False

    def has_failed_fitting(self) -> bool:
        return False

    #######################
    # Remote only methods #
    #######################

    @property
    def model_id(self) -> int:
        raise Exception("Must call .fit before the model has an ID")

    def save(self, path: PathLike) -> None:
        # Don't try and save models that are not currently on Analyze
        raise Exception("Must call .fit before a model can be saved")

    def predict(
        self,
        data: DataLike,
        *,
        name: str,
        description: Optional[str],
        wait_until_complete: bool,
    ) -> Prediction:
        raise Exception("Must call .fit before a prediction can occur")

    def test(
        self,
        data: DataLike,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        wait_until_complete: bool = True,
    ) -> Test:
        raise Exception("Must call .fit before a test can occur")

    def model_info(self) -> ModelResponse:
        raise Exception("Must call .fit before obtaining model information")

    def url(self) -> str:
        raise Exception("Must call .fit before obtaining model url")

    def wait_until_fitted(self):
        raise Exception("Must call .fit before waiting for the model to be fitted")



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.create_db_data_set_request_data_origin import (
    CreateDbDataSetRequestDataOrigin,
)
from ..models.create_db_data_set_request_db_type import CreateDbDataSetRequestDbType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateDbDataSetRequest")

@attr.s(auto_attribs=True)
class CreateDbDataSetRequest:
    """  """
    name: str
    db_type: CreateDbDataSetRequestDbType
    query: str
    data_origin: CreateDbDataSetRequestDataOrigin
    description: Union[Unset, str] = UNSET
    username: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    host: Union[Unset, str] = UNSET
    port: Union[Unset, str] = UNSET
    db_name: Union[Unset, str] = UNSET
    custom_db_url: Union[Unset, bool] = False
    db_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        name =  self.name
        db_type = self.db_type.value

        query =  self.query
        data_origin = self.data_origin.value

        description =  self.description
        username =  self.username
        password =  self.password
        host =  self.host
        port =  self.port
        db_name =  self.db_name
        custom_db_url =  self.custom_db_url
        db_url =  self.db_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "name": name,
            "dbType": db_type,
            "query": query,
            "dataOrigin": data_origin,
        })
        if description is not UNSET:
            field_dict["description"] = description
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password
        if host is not UNSET:
            field_dict["host"] = host
        if port is not UNSET:
            field_dict["port"] = port
        if db_name is not UNSET:
            field_dict["dbName"] = db_name
        if custom_db_url is not UNSET:
            field_dict["customDbUrl"] = custom_db_url
        if db_url is not UNSET:
            field_dict["dbUrl"] = db_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        db_type = CreateDbDataSetRequestDbType(d.pop("dbType"))




        query = d.pop("query")

        data_origin = CreateDbDataSetRequestDataOrigin(d.pop("dataOrigin"))




        description = d.pop("description", UNSET)

        username = d.pop("username", UNSET)

        password = d.pop("password", UNSET)

        host = d.pop("host", UNSET)

        port = d.pop("port", UNSET)

        db_name = d.pop("dbName", UNSET)

        custom_db_url = d.pop("customDbUrl", UNSET)

        db_url = d.pop("dbUrl", UNSET)

        create_db_data_set_request = cls(
            name=name,
            db_type=db_type,
            query=query,
            data_origin=data_origin,
            description=description,
            username=username,
            password=password,
            host=host,
            port=port,
            db_name=db_name,
            custom_db_url=custom_db_url,
            db_url=db_url,
        )

        create_db_data_set_request.additional_properties = d
        return create_db_data_set_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ProblemType(str, Enum):
    CLASSIFICATION = "CLASSIFICATION"
    REGRESSION = "REGRESSION"
    CLUSTERING = "CLUSTERING"
    AUTOCLUSTERING = "AUTOCLUSTERING"
    FORECASTING = "FORECASTING"

    def __str__(self) -> str:
        return str(self.value)